:
# NAME:
#	only1.sh - simple mechanism to ensure only one process runs.
#
# DESCRIPTION:
#	This module, is used to ensure only one copy of a program is
#	running.  That can be either the existing one, or the new one.
#	If the variable Only1Runner is not 1 or 0, it is assumed to be
#	a command to evaluate to decide the matter.
#
#	If 'atexit.sh' has been loaded, we use atexit to schedule
#	removal of the lock file ("Only1PidF"), otherwise we just use trap 0.
#
# BUGS:
#	Automagic removal of the lock file will not happen if the process
#	execs another command.
#	
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: only1.sh,v 1.22 2009/11/02 22:16:38 sjg Exp $
#
#	@(#) Copyright (c) 1995-2002 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

Myname=${Myname:-$0}
case "$Myname" in
*/*)	Myname=`basename $Myname .sh`;;
esac

if test x$Only1PidF = x; then
	test -d /var/run && Only1PidDir=${Only1PidDir:-${PIDDIR:-/var/run}}
	Only1PidDir=${Only1PidDir:-${PIDDIR:-/tmp}}

	PidF=$Only1PidDir/.$Myname

	case "$Only1Args" in
	none)	;;
	*)
		# use the args to further qualify...
		for i in $*
		do
			case $i in
			*/*)
				case "$Only1dirArgs" in
				none)	break;;
				full)	i=`echo $i | tr / _`;;
				*)	i=`basename $i`;;
				esac
			esac
			PidF="${PidF}_$i"
		done
		;;
	esac
	PidF=${PidF}.pid
else
	PidF=$Only1PidF
fi

Only1Runner=${Only1Runner:-0}	# first gets to run, else last only

lock=${PidF}
tlock=$lock.$$

# can you believe that ln on Solaris defaults to
# overwriting an existing file!!!!!
test -x /usr/xpg4/bin/ln && LN=/usr/xpg4/bin/ln

Only1Error() {
        echo "$Myname[$$]: $*" >&2
        exit 1
}
                                
lpid=x
while :
do
	echo $$ > $tlock
        test -s $tlock || Only1Error "cannot create temp lock '$tlock'"
	${LN:-ln} $tlock $lock > /dev/null 2>&1 && break
	rm -f $tlock		# avoid mess
	RPid=`test -s $lock && cat $lock`

	case "$RPid" in
	$$)	break;;
	"")	# an empty lock file should never happen
		rm -f $lock; test -f $lock || continue
		Only1Error "cannot remove empty lock '$lock'"
		;;
	esac
	# we can't simply use return code, as permission denied means
	# locker is alive... watch out for internationalization!
	psts=`LANG=C LC_ALL=C kill -0 $RPid 2>&1`
	case "$psts" in
	*:*such*p*)
		rm -f $lock; test -f $lock || continue
		Only1Error "cannot remove old lock '$lock' held by [$RPid]"
		;;	    
	esac
	# ok a valid lock exists, and it isn't us
	case "$RPid" in
	$lpid)
		Only1Error "cannot take lock '$lock' held by [$lpid]"
		;;
	esac
	lpid=$RPid
	case "$Only1Runner" in
	0)	exit 0;;
	[1-9])	kill $RPid 2>/dev/null && { sleep 5; kill -9 $RPid 2>/dev/null; };;
	*)	eval $Only1Runner $RPid;;
	esac
done
rm -f $tlock

case "$Only1Cleanup" in
none)	;;
"")
	# use atexit.sh if you want multiple things to happen
	case "${at_exit:+atexit}" in
	"")	trap "rm -f $PidF" 0;;
	*)	atexit "rm -f $PidF";;
	esac
	;;
*)	eval $Only1Cleanup $PidF;;
esac
# now do your thing...
